require 'assert'
require 'mr/after_commit/fake_record'

require 'much-plugin'
require 'mr/after_commit'
require 'mr/fake_record'

module MR::AfterCommit::FakeRecord

  class UnitTests < Assert::Context
    desc "FakeRecord"
    subject{ MR::AfterCommit::FakeRecord }

    should "use much-plugin" do
      assert_includes MuchPlugin, subject
    end

  end

  class MixinTests < UnitTests
    desc "when mixed in"
    setup do
      @fake_record_class = Class.new do
        include MR::AfterCommit::FakeRecord
      end
    end
    subject{ @fake_record_class }

    should "be an MR record" do
      assert_includes MR::FakeRecord, subject
    end

    should "use the records procs methods mixin" do
      assert_includes MR::AfterCommit::RecordProcsMethods, subject
    end

  end

  class InitTests < MixinTests
    desc "and init"
    setup do
      @fake_record = @fake_record_class.new

      i = 0
      @fake_record.add_after_commit_proc(:save){   @save_call_num   = i += 1 }
      @fake_record.add_after_commit_proc(:create){ @create_call_num = i += 1 }
      @fake_record.add_after_commit_proc(:update){ @update_call_num = i += 1 }

      @destroy_called = false
      @fake_record.add_after_commit_proc(:destroy){ @destroy_called = true }
    end
    subject{ @fake_record }

    should have_imeths :called_after_commit_procs

    should "call and clear its save and create procs when a new fake record is saved" do
      subject.save!

      assert_equal 1, @create_call_num
      assert_equal 2, @save_call_num
      assert_nil @update_call_num
      assert_false @destroy_called

      assert_empty subject.after_commit_procs(:create, :save)
      assert_not_empty subject.after_commit_procs(:update, :destroy)

      assert_equal 1, subject.called_after_commit_procs(:create).size
      assert_equal 1, subject.called_after_commit_procs(:save).size
      assert_equal 0, subject.called_after_commit_procs(:update).size
      assert_equal 0, subject.called_after_commit_procs(:destroy).size
    end

    should "call and clear its save and update procs when an existing fake record is saved" do
      subject.id = Factory.integer # make it an existing
      subject.save!

      assert_equal 1, @update_call_num
      assert_equal 2, @save_call_num
      assert_nil @create_call_num
      assert_false @destroy_called

      assert_empty subject.after_commit_procs(:update, :save)
      assert_not_empty subject.after_commit_procs(:create, :destroy)

      assert_equal 1, subject.called_after_commit_procs(:update).size
      assert_equal 1, subject.called_after_commit_procs(:save).size
      assert_equal 0, subject.called_after_commit_procs(:create).size
      assert_equal 0, subject.called_after_commit_procs(:destroy).size
    end

    should "call and clear its destroy procs after commit on destroy" do
      subject.destroy

      assert_true @destroy_called
      assert_nil @save_call_num
      assert_nil @create_call_num
      assert_nil @update_call_num

      assert_empty subject.after_commit_procs(:destroy)
      assert_not_empty subject.after_commit_procs(:create, :update, :save)

      assert_equal 1, subject.called_after_commit_procs(:destroy).size
      assert_equal 0, subject.called_after_commit_procs(:update).size
      assert_equal 0, subject.called_after_commit_procs(:save).size
      assert_equal 0, subject.called_after_commit_procs(:create).size
    end

  end

end
